#include "vector3d.h"
#include "map.h"
#include "gameobject.h"
#include <allegro.h>
#include <vector>
#include <map>
#include <string>

class HoverCraft; // forward declartation
class AiWaypoint;

class Manager {
	public:
		static const int VirtualWidth = 640*8;
		static const int VirtualHeight = VirtualWidth / 2;
		Manager(bool editMode); // default constructor - allocs bitmap
		void initLevel(int n);
		~Manager();
		void mainLoop();
		void preview(int l);
		BITMAP *getSprite(int n);
		SAMPLE *getSample(int n);
		Map *getMap() { return &map; }
		void addObject(HoverCraft *obj);
		void addObject(GameObject *obj);
		// Find the nearest object to obj which is not obj itself.
		GameObject *findNearest(GameObject *obj);
		bool isEditMode() { return editMode; }
		long getTime() { return timems; }
		int getNumCheckpoints() { return numCheckpoints; }
		void setNumCheckpoints(int n) { numCheckpoints = n; }
		int getNumWaypoints() { return waypoints.size(); }
		int getNumLaps() { return numLaps; }
		AiWaypoint * getWaypoint(int n) { return waypoints[n]; }
		void sortPlaces(); // Called to re-sort the vehicles list
		void freezeOpponents(HoverCraft *h);
		void showBigMessage(const std::string &msg);
		
		typedef std::vector<GameObject *> GameObjectList;
		typedef GameObjectList::iterator GameObjectIterator;
		// Use a std::multimap so we don't actually need to sort it, as it's auto sorted.
		typedef std::multimap <Component, GameObject *> RenderList;
		
		bool musicOn;
		
	private:
		BITMAP *backdrop;
		BITMAP *buffer;
		DATAFILE *df;
		Map map;
		View view;
		int offsetx, offsety; // viewport offsets.
		int level;
		bool editMode;
		long timems; // Time, in milliseconds.
		long timens; // Time in nanoseconds 
		int numCheckpoints;
		int numLaps;
		std::string bigMessage;
		Component messageTime;
		
		void loadData();
		void clearObjects();
		void render();
		void handleLogic(Component quantum);
		void loadTrack();
		void saveTrack();
		void initVehicles();
		void handleCollision(HoverCraft *veh, GameObject *obj);
		void findCollisions();
		
		GameObjectList gameObjects;
		GameObjectList newObjects; // storing new ones per tick...
		GameObjectList deadList;
		typedef std::vector<HoverCraft *> VehicleList;
		VehicleList vehicles;
		typedef std::vector<AiWaypoint*> Waypoints;
		Waypoints waypoints;
		
		RenderList renderList;
		GameObject * trackObject; // Which object the view tracks
		HoverCraft * playerObject;
};

